# This file is a portion of the Red Hat Network Panel Applet
#
# Copyright (C) 1999-2002 Red Hat, Inc. All Rights Reserved.
# Distributed under GPL version 2.
#
# Author: Chip Turner
#
# $Id: rhn_applet_model.py,v 1.17 2002/09/03 07:23:11 gafton Exp $

import string
import os
import time

import gettext
import rhn_utils

from rhn_applet_rpm import rhnAppletRPM
from rhn_applet_rpc import rhnAppletRPC

from rhn_utils import rhnAppletException


GETTEXT_DOMAIN = "rhn-applet"
gettext.bindtextdomain (GETTEXT_DOMAIN, "/usr/share/locale")
gettext.textdomain (GETTEXT_DOMAIN)
try:
    gettext.install(GETTEXT_DOMAIN, "/usr/share/locale", 1)
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode
except AttributeError: # older gettext modules don't have an install() method
    from gettext import _

class rhnAppletModel:
    def __init__(self, refresh_callback, url, uuid, mode = "gui", cert = None):
        self.__refresh_callback__ = refresh_callback
        self.__url__ = url
        self.__uuid__ = uuid

        self.__proxy__ = ''
        self.__proxy_username__ = ''
        self.__proxy_password__ = ''
        
        self.__arch__ = os.uname()[4]

        self.__needed_packages__ = []
        self.__ignored_packages__ = {}
        self.__release__ = None
        self.__kernel_running__ = None
        self.__rpc_server__ = None

        self.__rpmdb__ = None

        self.__last_rpm_change_number__ = 0
        self.__last_rpc_change_number__ = 0
        # public data member
        self.change_number = 0

        if not mode in ('gui', 'tui'):
            raise _("invalid rhnAppletModel mode %s") % mode
            
        self.__mode__ = mode
        self.__cert__ = cert

    def set_proxy(self, proxy, u, p):
        self.__proxy__ = proxy
        self.__proxy_username__ = u
        self.__proxy_password__ = p

        if self.__rpc_server__:
            self.__rpc_server__.set_proxy(proxy, u, p)

    def is_package_ignored(self, name):
        if self.__ignored_packages__.has_key(name):
            return 1
        else:
            return 0
    def add_ignored_package(self, name):
        self.__ignored_packages__[name] = 1
    def remove_ignored_package(self, name):
        if self.__ignored_packages__.has_key(name):
            del self.__ignored_packages__[name]
    def ignored_package_list(self):
        return self.__ignored_packages__.keys()

    def installed_package(self, name):
        rpm = self.__rpmdb__

        return rpm.find_latest_by_name(name)

    def needed_packages(self):
        return self.__needed_packages__, self.__ignored_packages__.keys()

    def alerts(self):
        if not self.fully_initialized():
            return []

        ret = []
        # find the latest version of the package that has the same
        # name as the package that the currently running kernel
        # belongs to

        if len(self.__kernel_running__):
            latest_kernel = self.__rpmdb__.find_latest_by_name(self.__kernel_running__["name"])
            if not latest_kernel["epoch"]:
                latest_kernel["epoch"] = None

            if latest_kernel != self.__kernel_running__:
                a = rhnAppletAlert('kernel', self.__kernel_running__, latest_kernel)
                ret.append(a)
        else:
            # not running a kernel in RPM format?  not much we can do
            a = rhnAppletAlert('unsupported-kernel', os.uname()[2])
            ret.append(a)
            pass

        needed_up2date = None
        for pkg in self.__needed_packages__:
            if pkg["name"] != "up2date":
                continue

            needed_up2date = pkg
            break
        
        if needed_up2date:
            a = rhnAppletAlert('up2date', self.installed_package("up2date"), needed_up2date)
            ret.append(a)

        return ret
    
    # if we're not initialized, checkin simply returns, letting
    # defaults control behavior
    
    def refresh(self, force=0):
        if not self.fully_initialized():
            return

        rpm = self.__rpmdb__
        latest_installed_packages = rpm.latest_installed_packages()
        latest_avail_packages = self.__rpc_server__.latest_packages()

        A = latest_avail_packages
        # format of elements of A is dict with keys:
        #   nvre, name, version, release, epoch, errata_advisory, errata_id


        I = latest_installed_packages
        # I is a hash of installed packages.  keys are package
        # names, values is LATEST version of package as a dict:
        #   nvre, name, version, version, epoch

        self.__needed_packages__ = []
        for latest in A:
            if not I.has_key(latest['name']):
                # package not on this system?  next!
                continue

            installed = I[latest["name"]]

            cmp = rpm.compare_vre(latest["version"],
                                  latest["release"],
                                  latest["epoch"],
                                  installed["version"],
                                  installed["release"],
                                  installed["epoch"])

            if cmp <= 0:
                # we have the same or a more recent
                continue

            self.__needed_packages__.append(latest)

        self.change_number = self.change_number + 1
        
    def fully_initialized(self):
        rpm = self.__rpmdb__
        
        if not self.__release__:
            releases = rpm.find_provides("redhat-release")
            if releases and len(releases):
                self.__release__ = releases[0]["version"]

        if self.__kernel_running__ == None:
            kernel_image = "/boot/vmlinuz-" + os.uname()[2]
            kernel_running = rpm.find_provides_by_file(kernel_image)

            if kernel_running and len(kernel_running):
                self.__kernel_running__ = {}
                for i in ("name", "version", "release", "epoch"):
                    self.__kernel_running__[i] = kernel_running[0][i]
            else:
                # no idea what kernel you're running
                print "could not figure out kernel"
                self.__kernel_running__ = [ ]

        if self.__release__ and not self.__rpc_server__:
            self.__rpc_server__ = rhnAppletRPC(self.__url__,
                                               self.__refresh_callback__,
                                               self.__uuid__,
                                               self.__release__,
                                               self.__arch__,
                                               self.__cert__,
                                               self.__proxy__,
                                               self.__proxy_username__,
                                               self.__proxy_password__)

        if self.__kernel_running__ != None and self.__rpc_server__ and rpm.latest_installed_packages():
            return 1

        return 0

    # change numbers are monotonically increasing values; idea
    # borrowed from the database world.  if something changed, we mark
    # it.  higher level components can watch for changes and act
    # accordingly.  fits in well with GUIs so we don't redraw too
    # often.
    
    def poll_change_numbers(self):
        changed = 0
        
        if self.__last_rpm_change_number__ != self.__rpmdb__.change_number:
            self.__last_rpm_change_number__ = self.__rpmdb__.change_number
            self.change_number = self.change_number + 1
            changed = 1
            
        if self.__last_rpc_change_number__ != self.__rpc_server__.change_number:
            self.__last_rpc_change_number__ = self.__rpc_server__.change_number
            self.change_number = self.change_number + 1
            changed = 1

        return changed

    def pkg_as_nvr(self, pkg):
        return string.join((pkg["name"], pkg["version"], pkg["release"]), "-")

class rhnAppletModelTUI(rhnAppletModel):
    def __init__(self, refresh_callback, url, uuid, cert = None):
        rhnAppletModel.__init__(self, refresh_callback, url, uuid, 'tui', cert)

        self.__rpmdb__ = rhnAppletRPM(fatal = 1)
        self.__rpmdb__.refresh()

        while not self.fully_initialized():
            pass

        self.__rpc_server__.refresh()

        self.change_number = 1
        self.poll_change_numbers()
        rhnAppletModel.refresh(self)
        
class rhnAppletModelGUI(rhnAppletModel):
    def __init__(self, refresh_callback, url, uuid, cert = None):
        rhnAppletModel.__init__(self, refresh_callback, url, uuid, 'gui', cert)
        self.__rpmdb__ = rhnAppletRPM(fatal = 0)

    def refresh(self, force=0):
        self.__rpmdb__.refresh(force)

        if not self.fully_initialized():
            print "ain't there yet"
            return
        
        self.__rpc_server__.refresh(force)

        if self.poll_change_numbers():
            rhnAppletModel.refresh(self)

class rhnAppletAlert:
    def __init__(self, type, *data):
        self.__type__ = type
        self.__data__ = data

    def is_unsupported_kernel_alert(self):
        if self.__type__ == 'unsupported-kernel':
            return 1
        return 0
    def is_kernel_alert(self):
        if self.__type__ == 'kernel':
            return 1
        return 0
    def is_up2date_alert(self):
        if self.__type__ == 'up2date':
            return 1
        return 0

    def alert_data(self):
        return self.__data__
